const electron = require('electron');
const { app, ipcMain } = electron;
const { shell } = require('electron');

// require and configure dotenv
require('dotenv').config();

// SET ENV
//process.env.NODE_ENV = 'production';

const DAWProjectSearcher = require('./daw-project-searcher').DAWProjectSearcher;
const WindowController = require('./window-controller');
const DatabaseController = require('./database-controller');
const AuthenticationController = require('./authentication-controller');

let dawProjectSearcher = new DAWProjectSearcher();
let databaseController = new DatabaseController();
let authenticationController = new AuthenticationController();
let windowController;

// Listen for app to be ready
app.on('ready', function () {
    windowController = new WindowController(1920, 1080);
    windowController.createMainWindow_Login();
});

// called when user presses login button
ipcMain.on('login', function(e, userID, userPassword){
    if(authenticationController.tryLogin(userID, userPassword)){
        // success, show the main page
        windowController.showMainWindow();

        // TODO store username in database if user wanted to be remembered?

        return; // exit function
    }
    // TODO send login-failed message to main page
});

 // reads the user's search paths from the database and updates the main window if any were found
ipcMain.on('readusersearchpaths', function(e){
    var userSearchDirs = databaseController.readUserSearchDirectories();
    if(userSearchDirs == null){
        return;
    }
    // if any entries were read, send to the main window
    windowController.mainWindow.webContents.send('updateusersearchpaths', userSearchDirs.directories);
});

// creates or updates the user search paths entry in the database with a provided string path
// @var filepath - full path string
ipcMain.on('createupdateusersearchpaths', function(e, filePath){
    databaseController.addUserSearchDirectory(filePath);
    // tell the renderer to pull the new data from the database, if there were changes
    var newUserSearchDirectories = databaseController.readUserSearchDirectories();
    windowController.mainWindow.webContents.send('updateusersearchpaths',newUserSearchDirectories.directories);

});

// Catch getprojectdetail 
// Called when user wants to view a project's details in a detail window
ipcMain.on('getprojectdetail', function (e, projectPath) {
    windowController.createDetailWindow(databaseController.readProject(projectPath));
});

// Catch openitem 
// Called when user requests to open a full path location in system explorer
// TODO Debug Windows, Linux functionality
ipcMain.on('openitem', function (e, fullPath) {
    var preparedPath = fullPath;
    // TODO decide if the paths should be reformatted to fit runtime OS's format -
    // TODO could add 'Host OS' data field to scanned project, or a host system ID, to check if we're on the system that the original project was scanned from.
    // TODO and/or, handle errors when path can't be shown in folder

    // switch (process.platform) {
    //     case 'win32':
    //         // TODO Debug and test windows paths
    //         //preparedPath = fullPath.replace(/\//g, '\\');
    //         break;
    //     case 'linux':
    //         preparedPath = "\"" + fullPath + "\"";
    //         break;
    //     default:
    //         preparedPath = fullPath;
    // }
    console.log('Main thread - opening in system file browser', preparedPath);
    shell.showItemInFolder(preparedPath);
});

// Catch cleardb
// Deletes all content from the local database
// TODO allow clearing of specific types of data     
ipcMain.on('cleardb', function(e){
    databaseController.clearLocalDatabase();
});

// Performs the scan on array of directory paths to be searched
// On completion, updates the database and main window with the discovered DAW projects
ipcMain.on('startscan', function (e, directories) {
    let projects = [];
    directories.forEach(directory =>{
        projects = projects.concat(dawProjectSearcher.getDAWProjects(directory));
    });
    
    projects.forEach(project => {
        //console.log(project);
        databaseController.createProject(project.dir.fullPath, project);
    });
    
     // TODO could have mainWindow refresh and get the data itself
    windowController.mainWindow.webContents.send('scan:complete:allentries', projects);
});